@@ -68,10 +68,8 @@ class MLaunchTool(BaseCmdLineTool):
         self.hostname = socket.gethostname()
 
 
-    def run(self):
-        # don't call BaseCmdLineTool.run(), we pass args ourselves
-        self.args, self.unknown_args = self.argparser.parse_known_args()
-        self.args = vars(self.args)
+    def run(self, arguments=None):
+        BaseCmdLineTool.run(self, arguments, get_unknowns=True)
 
         # load or store parameters
         if self.args['restart']:
