@@ -8,14 +8,16 @@ import uuid
 import glob
 import cPickle
 import types
+import inspect
 from copy import copy
+from mtools import __version__
 
 try:
     import matplotlib.pyplot as plt
-    from matplotlib.dates import DateFormatter
+    from matplotlib.dates import DateFormatter, date2num
     from matplotlib.lines import Line2D
     from matplotlib.text import Text
-    from mtools.mplotqueries.plottypes import DurationPlotType, EventPlotType, RangePlotType, RSStatePlotType
+    import mtools.mplotqueries.plottypes as plottypes
 except ImportError:
     raise ImportError("Can't import matplotlib. See https://github.com/rueckstiess/mtools/blob/master/INSTALL.md for instructions how to install matplotlib or try mlogvis instead, which is a simplified version of mplotqueries that visualizes the logfile in a web browser.")
 
@@ -35,7 +37,8 @@ class MPlotQueriesTool(LogFileTool):
         self.argparser.description='A script to plot various information from logfiles. ' \
             'Clicking on any of the plot points will print the corresponding log line to stdout.'
 
-        self.plot_types = [DurationPlotType, EventPlotType, RangePlotType, RSStatePlotType]
+        # import all plot type classes in plottypes module
+        self.plot_types = [c[1] for c in inspect.getmembers(plottypes, inspect.isclass)]
         self.plot_types = dict((pt.plot_type_str, pt) for pt in self.plot_types)
         self.plot_instances = []
 
@@ -44,16 +47,20 @@ class MPlotQueriesTool(LogFileTool):
         self.argparser.add_argument('--ns', action='store', nargs='*', metavar='NS', help='namespaces to include in the plot (default=all)')
         self.argparser.add_argument('--logscale', action='store_true', help='plot y-axis in logarithmic scale (default=off)')
         self.argparser.add_argument('--overlay', action='store', nargs='?', default=None, const='add', choices=['add', 'list', 'reset'])
-        self.argparser.add_argument('--type', action='store', default='duration', choices=self.plot_types.keys(), help='type of plot (default=duration)')
-        
-        mutex = self.argparser.add_mutually_exclusive_group()
-        mutex.add_argument('--group', help="specify value to group on. Possible values depend on type of plot. All basic plot types can group on 'namespace', 'operation', 'thread', range plots can additionally group on 'log2code'.")
-        mutex.add_argument('--label', help="instead of specifying a group, a label can be specified. Grouping is then disabled, and the single group for all data points is named LABEL.")
+        self.argparser.add_argument('--type', action='store', default='duration', choices=self.plot_types.keys(), help='type of plot (default=duration)')        
+        self.argparser.add_argument('--group', help="specify value to group on. Possible values depend on type of plot. All basic plot types can group on 'namespace', 'operation', 'thread', range plots can additionally group on 'log2code'.")
+
+        # mutex = self.argparser.add_mutually_exclusive_group()
+        # mutex.add_argument('--label', help="instead of specifying a group, a label can be specified. Grouping is then disabled, and the single group for all data points is named LABEL.")
 
         self.legend = None
 
-    def run(self):
-        LogFileTool.run(self)
+        # store logfile ranges
+        self.logfile_ranges = []
+
+
+    def run(self, arguments=None):
+        LogFileTool.run(self, arguments, get_unknowns=True)
 
         self.parse_loglines()
         self.group()
@@ -89,21 +96,24 @@ class MPlotQueriesTool(LogFileTool):
 
         # create generator for logfile(s) handles
         if type(self.args['logfile']) != types.ListType:
-            logfiles = [self.args['logfile']]
+            self.logfiles = [self.args['logfile']]
         else:
-            logfiles = self.args['logfile']
+            self.logfiles = self.args['logfile']
             
-        if len(logfiles) > 1:
+        if len(self.logfiles) > 1:
             multiple_files = True
             self.args['group'] = 'filename'
         
-        plot_instance = self.plot_types[self.args['type']](args=self.args)
+        plot_instance = self.plot_types[self.args['type']](args=self.args, unknown_args=self.unknown_args)
         
-        for logfile in logfiles:
+        for logfile in self.logfiles:
+            start = None
 
             for line in logfile:
                 # create LogLine object
                 logline = LogLine(line)
+                if not start:
+                    start = logline.datetime
 
                 if multiple_files:
                     # amend logline object with filename for group by filename
@@ -130,11 +140,16 @@ class MPlotQueriesTool(LogFileTool):
                     line_accepted = True
                     plot_instance.add_line(logline)
 
+            end = logline.datetime
+
+            # store start and end for each logfile
+            self.logfile_ranges.append( (start, end) )
+
         self.plot_instances.append(plot_instance)
 
         # close files after parsing
         if sys.stdin.isatty():
-            for f in logfiles:
+            for f in self.logfiles:
                 f.close()
 
 
@@ -231,6 +246,7 @@ class MPlotQueriesTool(LogFileTool):
         print "%5s  %s" % ("1-9", "toggle visibility of individual plots 1-9")
         print "%5s  %s" % ("0", "toggle visibility of all plots")
         print "%5s  %s" % ("-", "toggle visibility of legend")
+        print "%5s  %s" % ("f", "toggle visibility of footnote")
         print "%5s  %s" % ("g", "toggle grid")
         print "%5s  %s" % ("l", "toggle log/linear y-axis")
         print "%5s  %s" % ("q", "quit mplotqueries")
@@ -281,13 +297,21 @@ class MPlotQueriesTool(LogFileTool):
                 self.toggle_artist(self.legend)
                 plt.gcf().canvas.draw()
 
+        if event.key == 'f':
+            self.toggle_artist(self.footnote)
+            plt.gcf().canvas.draw()
+
 
     def plot(self):
         self.artists = []
-        axis = plt.subplot(111)
+        plt.figure(figsize=(12,8), dpi=100, facecolor='w', edgecolor='w')
+        axis = plt.subplot(111, )
 
+        ylabel = ''
         for i, plot_inst in enumerate(sorted(self.plot_instances, key=lambda pi: pi.sort_order)):
-            self.artists.extend(plot_inst.plot(axis, i))
+            self.artists.extend(plot_inst.plot(axis, i, len(self.plot_instances)))
+            if hasattr(plot_inst, 'ylabel'):
+                ylabel = plot_inst.ylabel
             
         self.print_shortcuts()
 
@@ -298,12 +322,20 @@ class MPlotQueriesTool(LogFileTool):
         for label in axis.get_xticklabels():  # make the xtick labels pickable
             label.set_picker(True)
 
-        # log y axis
+        # set xlim from min to max of logfile ranges
+        xlim_min = min([dt[0] for dt in self.logfile_ranges])
+        xlim_max = max([dt[1] for dt in self.logfile_ranges])
+        axis.set_xlim(date2num([xlim_min, xlim_max]))
+
+        # ylabel for y axis
         if self.args['logscale']:
-            axis.set_yscale('log')
-            axis.set_ylabel('query duration in ms (log scale)')
-        else:
-            axis.set_ylabel('query duration in ms')
+            ylabel += ' (log scale)'
+        axis.set_ylabel(ylabel)
+
+        # title and mtools link
+        axis.set_title(', '.join([l.name for l in self.logfiles]))
+        plt.subplots_adjust(bottom=0.15, left=0.1, right=0.95, top=0.95)
+        self.footnote = plt.annotate('created with mtools v%s: https://github.com/rueckstiess/mtools' % __version__, (10, 10), xycoords='figure pixels', va='bottom', fontsize=8)
 
         handles, labels = axis.get_legend_handles_labels()
         if len(labels) > 0:
@@ -311,7 +343,6 @@ class MPlotQueriesTool(LogFileTool):
 
         plt.gcf().canvas.mpl_connect('pick_event', self.onpick)
         plt.gcf().canvas.mpl_connect('key_press_event', self.onpress)
-
         plt.show()
 
 
